// gcc -o uberflate_test.exe uberflate_test.c uberflate.c

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "uberflate.h"



void std_err(void) {
    perror("\nError");
    exit(1);
}



int main(int argc, char *argv[]) {
    int             insz    = 0,
                    outsz   = 0;
    unsigned char   *in     = NULL,
                    *out    = NULL;

    time_t  benchmark;
    FILE    *fd;
    int     c,
            zlib    = 0;
    char    *in_fname,
            *out_fname;

    if(argc < 3) {
        fprintf(stderr,
            "\n"
            "Usage: %s <input_file> <output_file> [zlib]\n"
            "\n"
            "use - for stdin/stdout\n"
            "by default this test tool produces a deflate stream, add a 1 to the command\n"
            "line for creating a zlib stream (header+deflate+adler32)\n"
            "\n", argv[0]);
        exit(1);
    }

    in_fname  = argv[1];
    out_fname = argv[2];
    if(argc > 3) zlib = atoi(argv[3]);

    fprintf(stderr, "- input file                %s\n", in_fname);
    if(!strcmp(in_fname, "-")) {
        for(insz = 0;; insz++) {
            c = fgetc(stdin);
            if(c < 0) break;
            in = realloc(in, insz + 1);
            if(!in) std_err();
            in[insz] = c;
        }
    } else {
        fd = fopen(in_fname, "rb");
        if(!fd) std_err();
        fseek(fd, 0, SEEK_END);
        insz = ftell(fd);
        fseek(fd, 0, SEEK_SET);
        in = malloc(insz);
        if(!in) std_err();
        insz = fread(in, 1, insz, fd);
        fclose(fd);
    }

    outsz = UBERFLATE_MAXZIPLEN(insz);
    if(outsz < insz) outsz = insz;
    out   = malloc(outsz);
    if(!out) std_err();

    fprintf(stderr, "- max estimated output size 0x%08x / %d\n", outsz, outsz);
    fprintf(stderr, "- input size                0x%08x / %d\n", insz, insz);

    benchmark = time(NULL);
    outsz = uberflate(
        in,
        insz,
        out,
        outsz,
        zlib);

    fprintf(stderr, "- resulted output size      0x%08x / %d\n", outsz, outsz);
    fprintf(stderr, "- compression performed in  %d seconds\n", (int)(time(NULL) - benchmark));

    if(outsz < 0) {
        fprintf(stderr, "\n"
            "Error: compression failed, your output buffer is too small or there is a\n"
            "problem in the uberflate library\n");
        exit(1);
    }

    fprintf(stderr, "- output file               %s\n", out_fname);
    if(!strcmp(out_fname, "-")) {
        fwrite(out, 1, outsz, stdout);
    } else {
        fd = fopen(out_fname, "wb");
        if(!fd) std_err();
        fwrite(out, 1, outsz, fd);
        fclose(fd);
    }

    fprintf(stderr, "- done\n");
    free(in);
    free(out);
    return(0);
}


